#include <msp430g2231.h>
// http://www.hutasu.net/Mikrokontrollerit/MCU_launchpad_osa22.htm
// Ohjelmamuistin koko tavuina (byte) Flash/RAM = 446/12

#define LED_RED                      (BIT0)
#define LED_GREEN                   (BIT6)
#define INIT_LAUNCHPAD_LEDS  (P1DIR |= LED_RED+LED_GREEN)
#define LED_OFF                       (P1OUT &= ~(LED_RED + LED_GREEN))

// Moottorin ohjaukseen liittyvi defineit
#define FORWARD       1
#define BACKWARD     -1
#define STOP          0

// Moottorin ohjauspinnit
#define MOTOR_CTRL_PIN1   (BIT7)
#define MOTOR_CTRL_PIN2   (BIT6)
#define MOTOR_INIT             (P1DIR |= MOTOR_CTRL_PIN1+MOTOR_CTRL_PIN2)

// Globaalit muuttujat
volatile unsigned int button = 0; // asetetaan keskeytyksess

// Launchpad funktiot
inline void InitLaunchpad();

// Moottorinohjausfunktiot
inline void InitMotor();
void SetMotorDirection(signed int direction);

void main(void)
{
     signed int motorDirection = STOP;
     WDTCTL = WDTPW + WDTHOLD;    // watchdog pysytys
     InitLaunchpad();
     InitMotor();
     __enable_interrupt();
     while(1)
     {
          if(button == 1)
          {
               if(motorDirection == STOP)
               {
                    // Jos moottori ei liiku (ensimminen kynnistys)
                    // niin asetetaan moottori eteenpin
                    SetMotorDirection(FORWARD);
                    motorDirection = FORWARD;
               }
               else if(motorDirection == FORWARD)
               {
                    // jos moottori liikkui eteenpin, pysytetn se
                    // ja odotetaan hetki ja asetetaan moottori
                    // liikkumaan taaksepin
                    SetMotorDirection(STOP);
                    while(button++ < 10)
                    {
                         // keinotekoinen viive
                         __delay_cycles(100000);
                    }
                    SetMotorDirection(BACKWARD);
                    motorDirection = BACKWARD;
               }
               else
               {
                    // moottori liikkui taaksepin, pysytetn se
                    // ja odotetaan hetki ja asetetaan moottori
                    // liikkumaan eteenpin
                    SetMotorDirection(STOP);
                    while(button++ < 10)
                    {
                         // keinotekoinen viive
                         __delay_cycles(100000);
                    }
                    SetMotorDirection(FORWARD);
                    motorDirection = FORWARD;
               }
          // nollataan nappi ettei jd suorittamaan tt koko ajan
          button = 0;
          }
     }
}

inline void InitLaunchpad()
{
     INIT_LAUNCHPAD_LEDS; // Ledien alustus
     LED_OFF;  // LED1 ja LED2 pois plt
     // S2 (P1.3) kytkin tuloksi ylsvetovastuksella
     P1DIR &= ~BIT3;
     P1IE |= BIT3;
     P1IES |= BIT3;
     P1REN |= BIT3;
     P1OUT |= BIT3;
     // alustetaan CPU:n kello 1 Mhz
     BCSCTL1 = CALBC1_1MHZ;
     DCOCTL = CALDCO_1MHZ;
     // BCSCTL3 = XCAP_3;
     // kiteen alustus on kommentoitu koska sit ei nyt tarvita
}

inline void InitMotor()
{
     // alustetaan porttipinnit
     MOTOR_INIT;
     // Asetetaan ohjaus nollille
     P1OUT &= ~(MOTOR_CTRL_PIN1+MOTOR_CTRL_PIN2);
}

void SetMotorDirection(signed int direction)
{
     if(direction == FORWARD)
     {
          P1OUT |= MOTOR_CTRL_PIN2;
          P1OUT &= ~MOTOR_CTRL_PIN1;
     }
     else if(direction == BACKWARD)
     {
          P1OUT |= MOTOR_CTRL_PIN1;
          P1OUT &= ~MOTOR_CTRL_PIN2;
     }
     else
     {
          P1OUT &= ~(MOTOR_CTRL_PIN1+MOTOR_CTRL_PIN2);
     }
}

// P1 keskeytysrutiini
#pragma vector = PORT1_VECTOR
__interrupt void Port1_ISR(void)
{
     if(!(P1IN & BIT3))
     {
          /* P1IFG.3 */
          // Launchpadin nappia painettu
          button = 1;
          __delay_cycles(100000);
     }
     // nollataan liput
     P1IFG = 0;
}
